Attribute VB_Name = "modDropShadow"
Private Declare Function CreateCompatibleDC Lib "GDI32" (ByVal hDC As Long) As Long
Private Declare Function SelectObject Lib "GDI32" (ByVal hDC As Long, ByVal hObject As Long) As Long
Private Declare Function CreateCompatibleBitmap Lib "GDI32" (ByVal hDC As Long, ByVal nWidth As Long, ByVal nHeight As Long) As Long
Private Declare Function BitBlt Lib "GDI32" (ByVal hDCDest As Long, ByVal XDest As Long, ByVal YDest As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal hDCSrc As Long, ByVal XSrc As Long, ByVal YSrc As Long, ByVal dwRop As Long) As Long
Private Declare Function SetPixel Lib "GDI32" (ByVal hDC As Long, ByVal X As Long, ByVal Y As Long, ByVal crColor As Long) As Long
Private Declare Function GetPixel Lib "GDI32" (ByVal hDC As Long, ByVal X As Long, ByVal Y As Long) As Long
Private Declare Function DeleteDC Lib "GDI32" (ByVal hDC As Long) As Long
Private Declare Function DeleteObject Lib "GDI32" (ByVal hObject As Long) As Long

Private iShadowWidth As Integer

Private Sub CalculateShadowWidth(objPic As PictureBox, Optional ZoomFactor As Single = 1)

  If CInt(Right(frmPixbook.TabStrip1.SelectedItem.Key, 1)) > 2 Then
  
    iShadowWidth = 2.5 / iCvrHtMm * objPic.Container.ScaleHeight / ZoomFactor
    
  Else
  
    iShadowWidth = 2.5 / iPgHtMm * objPic.Container.ScaleHeight / ZoomFactor
    
  End If
  
End Sub

Public Sub DrawDropShadow(objPic As PictureBox)
'Procedure develops a "drop shadow" around a picturebox.
'Righthand and lower edge sections of the object container are stored in temporary Device Contexts and used as "background(s)" to create a shadow effect.
'These sections are added to the existing picturebox to create an integral effect.

  Dim bScaleMode As Boolean
  Dim iScaleMode As Integer
  
  If objPic.Container.ScaleMode <> vbPixels Then
  
    bScaleMode = True
    iScaleMode = objPic.Container.ScaleMode
    objPic.Container.ScaleMode = vbPixels
    
  End If
  
  Dim lDC As Long, rDC As Long
  Dim lBMP As Long, rBMP As Long
  Dim X As Long, Y As Long, uColor As Long
  
  CalculateShadowWidth objPic
  
  With objPic
  
    .Visible = False
    'Capture the right and lower "shadow sections"
    '- create a Device Context to store the lower edge container section
    lDC = CreateCompatibleDC(.hDC)
    lBMP = CreateCompatibleBitmap(.hDC, .Width + iShadowWidth, iShadowWidth)
    SelectObject lDC, lBMP
    '- capture the lower edge "shadow" section
    BitBlt lDC, 0, 0, .Width + iShadowWidth, iShadowWidth, .Container.hDC, .Left, .Top + .Height, vbSrcCopy
    
    '- create a Device Context to store the right edge container section
    rDC = CreateCompatibleDC(.hDC)
    rBMP = CreateCompatibleBitmap(.hDC, iShadowWidth, .Height + iShadowWidth)
    SelectObject rDC, rBMP
    '- capture the right edge "shadow" section
    BitBlt rDC, 0, 0, iShadowWidth, .Height + iShadowWidth, .Container.hDC, .Left + .Width, .Top, vbSrcCopy
    
    '- create a Device Context to store the generated image
    genDC = CreateCompatibleDC(.hDC)
    genBMP = CreateCompatibleBitmap(.hDC, .Width + iShadowWidth, .Height + iShadowWidth)
    SelectObject genDC, genBMP
    '- capture the original image
    BitBlt genDC, 0, 0, .Width - iShadowWidth, .Height - iShadowWidth, .hDC, 0, 0, vbSrcCopy
    
    'Enlarge objPic
    .Width = .Width + iShadowWidth
    .Height = .Height + iShadowWidth
    
    ' - Simulate a gradient shadow effect on the right edge...
    For X = 1 To iShadowWidth
    
      For Y = 0 To iShadowWidth - 1
      
        uColor = GetPixel(rDC, iShadowWidth - X, Y)
        SetPixel .hDC, .Width - X, Y, uColor
        
      Next Y
      
      For Y = iShadowWidth To iShadowWidth + 3
      
        uColor = GetPixel(rDC, iShadowWidth - X, Y)
        If X + Y <= .Height Then SetPixel .hDC, .Width - X, Y, uMask(3 * X * (Y - (iShadowWidth - 1)), uColor)
        
      Next Y
      
      For Y = iShadowWidth + 4 To .Height - 5
      
        uColor = GetPixel(rDC, iShadowWidth - X, Y)
        If X + Y <= .Height + 1 Then SetPixel .hDC, .Width - X, Y, uMask(15 * X, uColor)
        
      Next Y
      
      For Y = .Height - 4 To .Height - 1
      
        uColor = GetPixel(rDC, iShadowWidth - X, Y)
        If X + Y <= .Height + 3 Then SetPixel .hDC, .Width - X, Y, uMask(-3 * X * (Y - .Height), uColor)
        
      Next Y
      
    Next X
    
    ' - Simulate a gradient shadow effect on the bottom edge...
    For Y = 1 To iShadowWidth
    
      For X = 0 To iShadowWidth - 1
      
        uColor = GetPixel(lDC, X, iShadowWidth - Y)
        SetPixel .hDC, X, .Height - Y, uColor
        
      Next X
      
      For X = iShadowWidth To iShadowWidth + 3
      
        uColor = GetPixel(lDC, X, iShadowWidth - Y)
        SetPixel .hDC, X, .Height - Y, uMask(3 * (X - (iShadowWidth - 1)) * Y, uColor)
        
      Next X
      
      For X = iShadowWidth + 4 To .Width - 4
      
        uColor = GetPixel(lDC, X, iShadowWidth - Y)
        If X + Y <= .Width Then SetPixel .hDC, X, .Height - Y, uMask(15 * Y, uColor)
        
      Next X
      
    Next Y
    
    .Visible = True
    .Refresh
    
  End With
  
  ' - Release the resources
  DeleteDC lDC
  DeleteObject lBMP
  DeleteDC rDC
  DeleteObject rBMP
  
  If bScaleMode Then objPic.Container.ScaleMode = iScaleMode
  
End Sub

Private Function uMask(lScale As Long, lColor As Long) As Long
'Function splits a color into its RGB components and transforms the color using a pseudo scale 0..255

  Dim r As Long, G As Long, B As Long
  
  r = lColor Mod 256
  G = (lColor \ 256) Mod 256
  B = (lColor \ 256 \ 256) Mod 256
  
  'Create Fade effect:
  r = r - r * lScale / 255 'pTransform(lScale, R)
  If r < 0 Then r = 0
  G = G - G * lScale / 255 'pTransform(lScale, G)
  If G < 0 Then G = 0
  B = B - B * lScale / 255 'pTransform(lScale, B)
  If B < 0 Then B = 0
  
  uMask = RGB(r, G, B)
  
End Function

Public Sub ClearDropShadow(objPic As PictureBox, iIndex As Integer, Optional ZoomFactor As Single = 1)

  Dim bScaleMode As Boolean
  Dim iScaleMode As Integer
  
  If objPic.Container.ScaleMode <> vbPixels Then
  
    bScaleMode = True
    iScaleMode = objPic.Container.ScaleMode
    objPic.Container.ScaleMode = vbPixels
    
  End If
  
  CalculateShadowWidth objPic, ZoomFactor
  
  With objPic
  
    .Width = .Width - iShadowWidth
    .Height = .Height - iShadowWidth
    
    If CInt(Right(frmPixbook.TabStrip1.SelectedItem.Key, 1)) > 2 Then
    
      If arrPicCvr(iIndex) = vbNullString Then .BackColor = .BackColor 'Overwrite drop shadow with background color
      
    Else
    
      If arrPic(iIndex) = vbNullString Then .BackColor = .BackColor 'Overwrite drop shadow with background color
      
    End If
    
    .Refresh
    
  End With
  
  If bScaleMode Then objPic.Container.ScaleMode = iScaleMode
  
End Sub
